(function() {
    var lastTime = 0;
    var vendors = ['ms', 'moz', 'webkit', 'o'];
    for(var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
        window.requestAnimationFrame = window[vendors[x]+'RequestAnimationFrame'];
        window.cancelAnimationFrame = 
          window[vendors[x]+'CancelAnimationFrame'] || window[vendors[x]+'CancelRequestAnimationFrame'];
    }
 
    if (!window.requestAnimationFrame)
        window.requestAnimationFrame = function(callback, element) {
            var currTime = new Date().getTime();
            var timeToCall = Math.max(0, 16 - (currTime - lastTime));
            var id = window.setTimeout(function() { callback(currTime + timeToCall); }, 
              timeToCall);
            lastTime = currTime + timeToCall;
            return id;
        };
 
    if (!window.cancelAnimationFrame)
        window.cancelAnimationFrame = function(id) {
            clearTimeout(id);
        };
}());

String.prototype.hashCode = function(){
    var hash = 0;
    var character = null;
    if (this.length === 0){
        return hash;
    }
    for (var i = 0; i < this.length; i++) {
        character = this.charCodeAt(i);
        hash = ((hash<<5)-hash)+character;
        hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
}

/** LatencyAnimation object
To use:
var blah = new LatencyAnimation();
blah.init(parentElment);
blah.start();
blah.stop();
*/
function LatencyAnimation(){
    this.animationInterval = null; /* keeps track of the interval used to call the step() method */
    this.animationTimeout = null; /* keeps track of the interval used to call the animate() method */
    this.animationFram = null; /* keep track of our requestAnimationFrame so we can cancel it */
    this.lastTime = new Date(); /* keep track of the timestamp of or previous step, so we can ensure we don't go too fast */ 

    this.displayCanvas = null; /* this canvas is visible to the user */
    this.displayCanvasContext = null; /* for ease of lookup */
    this.renderCanvas = null; /* this canvas exists off screen and out of the DOM and is used to speed up rendering */
    this.renderContext = null; /* for ease of lookup */
    this.alive = null; /* the array that is currently keeping track of the state of all the dots */
    this.canvasParent = document.getElementById("solutions"); /* DOM parent for ease of lookup and default value */
    this.size = 139; /* diameter of our dots */
    this.stepDelay = 0; /* time in ms between attempting to update the alive array via the step() method */
    this.iterations = 0;
    this.dots = new Array(); // pre render the dots and store as images
    
    this.parentWidth = this.canvasParent.offsetWidth;  /* How wide a space are we given at load time ? */
    this.parentHeight = this.canvasParent.offsetHeight;

    this.renderHeight = this.parentWidth >= this.parentHeight? this.parentWidth:this.parentHeight; /* how big the canvas needs to be to properly render the animation as a square */
    this.renderWidth = this.renderHeight;

    this.displayHeight = this.parentHeight; /* how much we display onscreen with the displayCanvas */
    this.displayWidth = this.parentWidth; /* generally, we want to fill the parent container */

    /* how many rows should be in the alive array */
    this.rows = Math.floor(this.renderHeight/(this.size+1));
     /* how many cells are in each row of the alive array */
    this.cells = Math.floor(this.renderWidth/(this.size+1)); 
    this.rowPadding = 0; /* if a seed is too small, we add empty rows above and below it to fill the space to make the array a square */
    /* rules for generations of dots in the alive array */
    this.births = [3,5,7];
    this.survivors = [3,4,5,7];
    this.generations = 5;

    this.colors=["#eeeeee","#f9c8c6","#fb9a95","#e1444c","#da2425","#e1444c","#fb9a95","#f2d5d5"];
    this.globalOpacity = 10; /* needed for fading a canvas in or out */
    this.displayCanvas = document.createElement( 'canvas' );

    this.seedList=[
[[0,0,0,1,0,0,0,0,0,0],[0,2,1,1,1,1,0,0,0,0],[1,1,3,4,2,1,1,0,0,0],[0,1,1,1,4,1,1,0,0,0],[0,1,2,0,1,1,1,1,0,0],[0,1,1,1,0,0,2,2,0,0],[0,0,2,2,0,0,1,1,1,0],[0,0,1,1,1,1,0,2,1,0],[0,0,0,1,1,4,1,1,1,0],[0,0,0,1,1,2,4,3,1,1],[0,0,0,0,1,1,1,1,2,0],[0,0,0,0,0,0,1,0,0,0]],
[[0, 0, 0, 0, 0, 0, 1, 0], [1, 1, 1, 1, 0, 0, 1, 0], [0, 1, 0, 0, 1, 1, 1, 1], [0, 1, 0, 0, 0, 0, 0, 0]],
[[0, 0, 0, 0, 0, 0, 1, 1], [0, 0, 1, 1, 1, 0, 1, 1], [1, 1, 0, 1, 1, 1, 0, 0], [1, 1, 0, 0, 0, 0, 0, 0]],
[[0, 0, 0, 0, 0, 0, 1, 1], [0, 1, 1, 1, 1, 1, 0, 0], [0, 0, 1, 1, 1, 1, 1, 0], [1, 1, 0, 0, 0, 0, 0, 0]],
[[0, 0, 0, 0, 0, 0, 1, 1], [1, 0, 0, 0, 1, 1, 1, 0], [0, 1, 1, 1, 0, 0, 0, 1], [1, 1, 0, 0, 0, 0, 0, 0]],
[[0, 0, 0, 0, 0, 0, 1, 1], [1, 0, 0, 1, 1, 0, 1, 1], [1, 1, 0, 1, 1, 0, 0, 1], [1, 1, 0, 0, 0, 0, 0, 0]],
[[0, 0, 0, 0, 1, 1, 1, 0], [1, 1, 1, 0, 1, 1, 1, 1], [1, 1, 1, 1, 0, 1, 1, 1], [0, 1, 1, 1, 0, 0, 0, 0]],
[[0, 0, 1, 1, 1, 1, 1, 1], [1, 0, 1, 1, 0, 1, 1, 1], [1, 1, 1, 0, 1, 1, 0, 1], [1, 1, 1, 1, 1, 1, 0, 0]],
[[0, 1, 0, 0, 0, 1, 1, 1], [1, 0, 1, 1, 0, 1, 1, 1], [1, 1, 1, 0, 1, 1, 0, 1], [1, 1, 1, 0, 0, 0, 1, 0]],
[[0, 1, 1, 0, 1, 1, 1, 1], [0, 1, 0, 1, 0, 0, 1, 0], [0, 1, 0, 0, 1, 0, 1, 0], [1, 1, 1, 1, 0, 1, 1, 0]],
[[1, 1, 1, 0, 1, 1, 1, 0], [0, 1, 0, 0, 1, 1, 1, 0], [0, 1, 1, 1, 0, 0, 1, 0], [0, 1, 1, 1, 0, 1, 1, 1]],
[[1, 1, 1, 0, 1, 1, 1, 1], [1, 1, 0, 1, 1, 1, 1, 0], [0, 1, 1, 1, 1, 0, 1, 1], [1, 1, 1, 1, 0, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 0, 1], [1, 1, 0, 1, 1, 0, 0, 1], [1, 0, 0, 1, 1, 0, 1, 1], [1, 0, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 1, 0], [0, 0, 1, 0, 1, 1, 0, 1], [1, 0, 1, 1, 0, 1, 0, 0], [0, 1, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 1, 0], [1, 0, 0, 1, 1, 0, 1, 1], [1, 1, 0, 1, 1, 0, 0, 1], [0, 1, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 1, 0], [1, 1, 1, 0, 1, 0, 0, 1], [1, 0, 0, 1, 0, 1, 1, 1], [0, 1, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 1, 0], [1, 1, 1, 0, 1, 1, 0, 1], [1, 0, 1, 1, 0, 1, 1, 1], [0, 1, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 0, 1, 0], [1, 1, 1, 1, 0, 0, 1, 1], [1, 1, 0, 0, 1, 1, 1, 1], [0, 1, 0, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 1, 0, 1], [1, 1, 1, 0, 0, 1, 0, 0], [0, 0, 1, 0, 0, 1, 1, 1], [1, 0, 1, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 1, 1, 0], [0, 0, 0, 0, 0, 1, 1, 0], [0, 1, 1, 0, 0, 0, 0, 0], [0, 1, 1, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 1, 1, 0], [0, 1, 0, 0, 1, 0, 1, 0], [0, 1, 0, 1, 0, 0, 1, 0], [0, 1, 1, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 0, 1, 1, 0], [1, 1, 1, 0, 0, 1, 1, 0], [0, 1, 1, 0, 0, 1, 1, 1], [0, 1, 1, 0, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 0, 0], [0, 0, 1, 1, 1, 1, 0, 0], [0, 0, 1, 1, 1, 1, 0, 0], [0, 0, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 0, 0], [1, 1, 0, 1, 1, 1, 1, 0], [0, 1, 1, 1, 1, 0, 1, 1], [0, 0, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 1, 1], [0, 0, 1, 1, 1, 0, 0, 0], [0, 0, 0, 1, 1, 1, 0, 0], [1, 1, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 1, 1], [0, 1, 1, 1, 1, 1, 0, 1], [1, 0, 1, 1, 1, 1, 1, 0], [1, 1, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 1, 1], [1, 1, 0, 1, 0, 1, 1, 0], [0, 1, 1, 0, 1, 0, 1, 1], [1, 1, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 0, 1, 1], [1, 1, 1, 1, 0, 0, 1, 1], [1, 1, 0, 0, 1, 1, 1, 1], [1, 1, 0, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 1, 0, 0], [0, 0, 1, 0, 0, 1, 1, 1], [1, 1, 1, 0, 0, 1, 0, 0], [0, 0, 1, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 1, 0, 0], [0, 1, 1, 1, 1, 1, 1, 0], [0, 1, 1, 1, 1, 1, 1, 0], [0, 0, 1, 1, 1, 1, 1, 1]],
[[1, 1, 1, 1, 1, 1, 1, 0], [0, 0, 1, 0, 1, 0, 1, 1], [1, 1, 0, 1, 0, 1, 0, 0], [0, 1, 1, 1, 1, 1, 1, 1]]
];

    
    /* flag for IE and other no-canvas browsers */
    this.supportsCanvas = !!(this.displayCanvas.getContext && this.displayCanvas.getContext('2d'));

    /* Add the animation to the DOM and seed it */
    this.init = function(cParent) {
        if(cParent) { /* cParent is a DOM object */
            this.canvasParent = cParent;
        }
        if(this.supportsCanvas) {
            this.displayCanvas.width = this.parentWidth;
            this.displayCanvas.height = this.displayHeight;
            this.displayCanvas.className = "nofade";
            this.displayCanvasContext = this.displayCanvas.getContext( '2d' );
            this.displayCanvasContext.globalCompositeOperation = 'copy';
            this.displayCanvas.id = "latency"; /* TODO allow for more than one id */
            this.renderCanvas = document.createElement( 'canvas' );
            this.renderContext = this.renderCanvas.getContext( '2d' );
       
            //this.rows = Math.floor(this.renderHeight/(this.size+1));
            /* how many cells are in each row of the alive array */
            var largerDimension = this.renderWidth > this.renderHeight ? this.renderWidth:this.renderHeight;
            this.cells = Math.floor(largerDimension/(this.size+1));
            this.rows = this.cells;

            // cache the dots we'll draw instead of drawing the path
            this.renderCanvas.width = this.size+1;
            this.renderCanvas.height = this.size+1;
            var dot = null;
            var centerX = Math.floor(this.size/2);
            var centerY = centerX;

            // pre-render dots and save as images to avoid recalculating
            for(var i=0,len=this.colors.length; i < len; i++){
                //this.renderContext.fillStyle = "#fff";
                //this.renderContext.clearRect(0,0,this.renderWidth,this.renderHeight);
                this.renderContext.fillStyle = this.colors[i];
                this.renderContext.beginPath();
                this.renderContext.arc( centerX,centerY, i<1?(this.size/8)+.5*this.size*(1/this.colors.length):(this.size/8)+.5*this.size*(i/this.colors.length), 0, 2 * Math.PI, false);
                //this.renderContext.closePath();
                this.renderContext.fill();
                // save canvas image as data url (png format by default)
                dot = new Image();
                dot.src = this.renderCanvas.toDataURL();
                this.dots[i] = dot;
            }
            // set the proper height and width once we are done pre-rendering dots       
            this.renderCanvas.width = this.renderWidth;
            this.renderCanvas.height = this.renderHeight;

            this.displayCanvas.style.display = "none"; /* canvas is exposed via the start method */
            this.canvasParent.appendChild(this.displayCanvas);
        } else {
          document.documentElement.className = document.documentElement.className + " noCanvas";      
        }
    }

    this.center = function(seed){
        var seedRows = seed.length;
        var seedCells = seed[0].length;
        var cellDiff = this.cells - seedCells;
        var rowDiff = this.rows - seedRows;
        var centeredSeed = new Array();
        if(cellDiff > 1){
            var cellPadding = Math.floor(cellDiff/2);
            var cellPaddingArray = new Array(cellPadding);
            for(var i=0; i<seedRows; i++){
                centeredSeed[i] = cellPaddingArray.concat(seed[i],cellPaddingArray);
            }   
        } else {
            centeredSeed = seed;
        }
        if(rowDiff >= 1) {
            this.rowPadding = Math.floor(rowDiff/2);           

            for(var i=0 ; i<=this.rowPadding; i++){
                centeredSeed.unshift(new Array(this.cells));
                if(centeredSeed.length != this.cells)
                centeredSeed.push(new Array(this.cells));
            }
        }
        this.alive = centeredSeed;
        this.rows = this.alive.length; /* correct for seed centering */
        this.cells = this.alive[0].length; /* correct for seed centering */
        return centeredSeed;
    }

    this.start = function(seed,skip,delay){
        if(!this.supportsCanvas){return;}
        var _this = this; /* closure required for setInterval */
        if(seed) {
            try {
                if(typeof seed === "object"){ /* assume a link was clicked.  TODO:  is this the only case? */
                    seed = /[\\?&]i=([^&#]*)/g.exec(seed.target.href)[1].hashCode()%this.seedList.length; 
                }
                else if(typeof seed != 'string') {
                    seed = +seed;
                }
            }
            catch(e) {
                seed = 0; /* fallback to 0 */  
            } 
            this.center(this.seedList[seed]);
        }
        else {
            seed = +Math.abs(document.getElementById("i").value.hashCode())%this.seedList.length;
            this.center(this.seedList[seed]);
        }
        if(typeof skip==='number' && skip > 0) {
           this.step(skip);
        }
        else {
           this.iterations = 0;
        }        
        if(delay){
            clearTimeout(this.animationTimeout);
            window.setTimeout(function(){_this.animate(new Date());},delay);
        }
        else {
            clearTimeout(this.animationTimeout);
            this.animate(new Date());
        }
        /* clear any properties that would prevent display and let css work */
        this.globalOpacity = 1; /* start at 10 to fade in */
        this.displayCanvas.style.display = "";
        this.displayCanvas.style.opacity = "";
    }

    this.stop = function(){
        this.clear();
        clearTimeout(this.animationTimeout);
        cancelAnimationFrame(this.animationFrame);
        this.iterations = 0;
        localStorage.setItem("iterations",this.iterations); /* restart at begining */
    }

    this.animate = function(timestamp) {
        var _this = this; /* closure required for requestAnimFrame */
        if (timestamp - this.lastTime < this.stepDelay){ /* if we have not gone at least as long as the stepDelay */
            cancelAnimationFrame(_this.animationFrame);
            _this.animationFrame = requestAnimationFrame(function(ts){/* queue up another attempt to render, up to 60 times per second */
                    _this.animate(ts); /* ts is the current timestamp and it gets passed by the requestAnimationFrame API */
                });  
            return;  /* stop here */
        } 
        this.lastTime = timestamp; /* keep track of our last render time */
        clearTimeout(this.animationTimeout);
        this.animationTimeout = setTimeout(function(){  /* set a timeout based upon stepDelay */
            cancelAnimationFrame(_this.animationFrame);
            _this.animationFrame = requestAnimationFrame(function(ts){_this.animate(ts);}); /* call our render function again */
           // _this.displayCanvasContext.clearRect(0,0,_this.parentWidth,_this.renderHeight); /* clear the displayCanvas */
            /* copy the off screen renderCanvas contents into the display canvas.  Center the contents vertically because they may have diff heights */
            _this.displayCanvasContext.drawImage(_this.renderCanvas, -Math.floor((_this.renderWidth-_this.displayWidth)/2)-2, -Math.floor((_this.renderHeight-_this.displayHeight)/2)-2);
            _this.step(); /* advance our alive array via the rules and update the off screen canvas - note that this can only happen 1 time per stepDelay */
         }, _this.stepDelay);
    }

    /* clear both the offscreen and onscreen canvas of dots */
    this.clear = function(){
        if(this.renderContext){
            this.renderContext.fillStyle = "#fff";
            this.renderContext.clearRect(0,0,this.renderWidth,this.renderHeight);
        }
        if(this.displayCanvasContext){
            this.displayCanvasContext.clearRect(0,0,this.renderWidth,this.renderHeight);
        }
    }

    this.step = function(skip) {
        // start with a clean slate
        this.renderContext.fillStyle = "#fff";
        this.renderContext.clearRect(0,0,this.renderWidth,this.renderHeight);

      // use a copy of the array because we can't update values in the alive array while still checking status
        var aliveCopy = null;
        var xWrapPlus = 1;
        var yWrapPlus = 1;
        var xWrapMinus = 1;
        var yWrapMinus = 1;
        var cellNeighbours = 0;

        if( document.documentElement.className.indexOf('loading')<0){
            this.globalOpacity = 0;//this.globalOpacity>0?this.globalOpacity-1:0;
        } else {
            this.globalOpacity=this.globalOpacity<10?this.globalOpacity+1:10;
        }
        if(this.globalOpacity==0) {
            this.stop();
            return;
        }
        this.renderContext.globalAlpha=this.globalOpacity/10;

        var iter = typeof skip === 'number'?skip:1; 
        for(var z=0; z < iter; z++){
        aliveCopy = new Array();
            for (var y=0;y<this.rows;y++) {
                yWrapPlus = y==(this.rows-1)?-1:y;
                yWrapMinus = y==0?this.rows:y;
                for (var x=0; x<this.cells;x++) {
                    xWrapPlus = x==(this.cells-1)?-1:x;
                    xWrapMinus = x==0?this.cells:x;
                    cellNeighbours = 0;
                    if (this.alive[y][xWrapMinus-1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[y][xWrapPlus+1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapPlus+1][x] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapMinus-1][x] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapMinus-1][xWrapMinus-1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapPlus+1][xWrapMinus-1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapPlus+1][xWrapPlus+1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (this.alive[yWrapMinus-1][xWrapPlus+1] == 1) {
                        cellNeighbours += 1;
                    }
                    if (aliveCopy[y] == undefined) {
                        aliveCopy[y] = [];
                    } 
                    aliveCopy[y][x] =  0;
                    if(this.alive[y][x] > 1){
                        aliveCopy[y][x] = (this.alive[y][x]+1)%this.generations;
                    } else {
                        if(!this.alive[y][x] && this.births.indexOf(cellNeighbours)!=-1){
                            aliveCopy[y][x] = 1;
                        } 
                        else if(this.alive[y][x] == 1) {
                            if(this.survivors.indexOf(cellNeighbours) != -1){
                                aliveCopy[y][x] = 1;
                            } else {
                                aliveCopy[y][x] = 2;
                            }                        
                        }
                    } 
                    if(typeof skip==='undefined'){ 
                        this.renderContext.drawImage(this.dots[aliveCopy[y][x]],x*(this.size+1),y*(this.size+1));
                    }
                }
            }
        this.alive = aliveCopy;
        this.iterations++;
        }
    }

}/* end LatencyAnimation Object */

var WolframAlphaLatencyAnimation = new LatencyAnimation();
// if we are comming from wolframalpha, get the number of iterations we need to skip
var startCycle = 0;
if(document.referrer.indexOf("wolframalpha.com") != -1){
    startCycle = +localStorage.getItem("iterations");
}
/*
WolframAlphaLatencyAnimation.init(document.getElementById("solutions"));
WolframAlphaLatencyAnimation.start(false,startCycle,0);
*/
// store iteration number for smoother page transitions
var iterationsStorage = function(e) {
    localStorage.setItem("iterations",WolframAlphaLatencyAnimation.iterations.toString());
};

// standard event binding
if (window.addEventListener) {
    window.addEventListener("unload", iterationsStorage);
} 
// IE8 event binding
else if (window.attachEvent) {
    window.attachEvent("onunload", iterationsStorage);
}
